//
//  UpnpDevice.m
//  uPnp2
//
//  Created by rk on 3/01/11.
//  Copyright 2011 Interchart. All rights reserved.
//

#import <Foundation/NSStream.h>
#import "UpnpDevice.h"




@implementation UpnpDevice

@synthesize delegate;
@synthesize _deviceType;
@synthesize _url;
@synthesize _readData;
@synthesize _deviceInfo;

NSString * const SCHEMA_SERVER_STRING = @"urn:schemas-upnp-org:device:MediaServer:1";
NSString * const SCHEMA_RENDERER_STRING = @"urn:schemas-upnp-org:device:MediaRenderer:1";



#pragma mark -
#pragma mark displayName property (readonly)
- (NSString *)displayName {
	NSString *result = @"UpnpDevice"; // default
	if (_url != nil) {
		result = [_url absoluteString];
	}
	if (_deviceInfo != nil) {
		result = _deviceInfo.friendlyName;
	}
	return result;
}

#pragma mark -
#pragma mark UpnpDeviceDelegate implementation
NSString *UpnpDeviceDisplayNameDidChangeValueNotification =
		@"UpnpDeviceDisplayNameDidChangeValueNotification";


- (void) upnpDeviceDisplayNameDidChangeValue {
	NSNotification *notification;
	notification = [NSNotification notificationWithName:
					UpnpDeviceDisplayNameDidChangeValueNotification
												 object:self];
	if ([self.delegate respondsToSelector: @selector(upnpDeviceDisplayNameDidChangeValue:)]) {
		[self.delegate upnpDeviceDisplayNameDidChangeValue:notification];
	}
}
#pragma mark -


- (id)init {
	self = [super init];
	return self;
}

- (void) dealloc {
	[super dealloc];
	[_readData release];
	_readData = nil;
	[_url release];
	_url = nil;
	[_deviceInfo release];
	_deviceInfo = nil;
}

- (void) fetchDeviceInfo {
	UpnpDevice *device = self;
	NSURL *deviceInfoUrl = device._url;
		
	NSURLRequest *request = [NSURLRequest 
							 requestWithURL:deviceInfoUrl 
							 cachePolicy:NSURLRequestReloadIgnoringCacheData 
							 timeoutInterval:30];
	// Clear out the existing connection if there is one
	if (_connectionInProgress) {
		[_connectionInProgress cancel];
		[_connectionInProgress release];
		_connectionInProgress = nil;
	}
	
	[_readData release];
	_readData = [[NSMutableData alloc] init];
	
	// Create and initiate the connection - non-blocking
	_connectionInProgress = [[NSURLConnection alloc] initWithRequest:request
															delegate:self
													startImmediately:YES];
}

//This method will be called several times as the data arrives
- (void) connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	[_readData appendData: data];
}

- (void) connectionDidFinishLoading:(NSURLConnection *)connection {
	[_connectionInProgress release];
	_connectionInProgress = nil;
	NSLog(@"finished receiving info data from device, %@", [self _requestResponse ]);
	UpnpDeviceInfo *deviceInfo = [[UpnpDeviceInfo alloc] initWithXml:self : _readData];
	_deviceInfo = deviceInfo;
	if (_deviceInfo) {
		NSString *typeSchema = _deviceInfo.deviceTypeSchema;
		if ([typeSchema isEqualToString:SCHEMA_RENDERER_STRING]) {
			_deviceType = Renderer;
		} else		
			if ([typeSchema isEqualToString:SCHEMA_SERVER_STRING]) {
				_deviceType = Server;
			} else {
				_deviceType = Undefined;
			}
		
		[self upnpDeviceDisplayNameDidChangeValue]; // notify delegate that display name is different now	 
	}
	NSLog(@"deviceInfo: %@", _deviceInfo);
	[_readData release];
	_readData = [[NSMutableData alloc] init]; // ready for next read
	for (UpnpAvService *service in _deviceInfo.services) {
		[service fetchXmlFromDevice:_url :service.scpdUrl];
	}
}

#pragma mark services property
-(NSMutableArray *)services {
	return [self _deviceInfo].services;
}


#pragma mark -

- (NSString *)_requestResponse {
	return [NSString stringWithFormat: @"response:\r\n%@\r\n",
			[NSString stringWithUTF8String:[_readData bytes]]];
}

@end
